@@ -182,7 +182,8 @@ void NV_API_CALL os_io_write_dword(
 void* NV_API_CALL os_map_kernel_space(
     NvU64 start,
     NvU64 size,
-    NvU32 mode
+    NvU32 mode,
+    NvU32 memType
 )
 {
     int map_mode;
@@ -396,18 +397,38 @@ NvS32 NV_API_CALL os_mem_cmp(
 }
 
 NvU8* NV_API_CALL os_mem_copy(
-    NvU8 *dst,
+    NvU8       *dst,
     const NvU8 *src,
-    NvU32 length
+    NvU32       length
 )
 {
-#if defined(NVCPU_X86_64)
-    uint32_t i;
-    for (i = 0; i < length; i++) dst[i] = src[i];
-    return dst;
-#else
-    return memcpy(dst, src, length);
-#endif
+    NvU8 *ret = dst;
+    NvU32 dwords, bytes = length;
+
+    if ((length >= 128) &&
+        (((NvUPtr)dst & 3) == 0) & (((NvUPtr)src & 3) == 0))
+    {
+        dwords = (length / sizeof(NvU32));
+        bytes = (length % sizeof(NvU32));
+
+        while (dwords != 0)
+        {
+            *(NvU32 *)dst = *(const NvU32 *)src;
+            dst += sizeof(NvU32);
+            src += sizeof(NvU32);
+            dwords--;
+        }
+    }
+
+    while (bytes != 0)
+    {
+        *dst = *src;
+        dst++;
+        src++;
+        bytes--;
+    }
+
+    return ret;
 }
 
 RM_STATUS NV_API_CALL os_memcpy_from_user(
@@ -435,12 +456,22 @@ RM_STATUS NV_API_CALL os_memcpy_to_user(
 }
 
 void* NV_API_CALL os_mem_set(
-    void *b,
-    NvU8  c,
-    NvU32 length
+    void  *dst,
+    NvU8   c,
+    NvU32  length
 )
 {
-    return memset(b, c, length);
+    NvU8 *ret = dst;
+    NvU32 bytes = length;
+
+    while (bytes != 0)
+    {
+        *(NvU8 *)dst = c;
+        dst = ((NvU8 *)dst + 1);
+        bytes--;
+    }
+
+    return ret;
 }
 
 char* NV_API_CALL os_string_copy(
@@ -466,9 +497,14 @@ NvU64 NV_API_CALL os_get_page_mask(void)
     return ~PAGE_MASK;
 }
 
-NvU64 NV_API_CALL os_get_system_memory_size(void)
+NvU8 NV_API_CALL os_get_page_shift(void)
 {
-    return ((NvU64)physmem * PAGE_SIZE) / RM_PAGE_SIZE;
+    return PAGE_SHIFT;
+}
+
+NvU64 NV_API_CALL os_get_num_phys_pages(void)
+{
+    return (NvU64)physmem;
 }
 
 NvU32 NV_API_CALL os_get_cpu_count(void)
